@@ -34,6 +34,9 @@ def append_dims(t, dims):
     shape = t.shape
     return t.reshape(*shape, *((1,) * dims))
 
+def l2norm(t):
+    return F.normalize(t, dim = -1)
+
 # u-vit related functions and modules
 
 class Upsample(nn.Module):
@@ -198,9 +201,9 @@ class LinearAttention(nn.Module):
         return self.to_out(out) + residual
 
 class Attention(nn.Module):
-    def __init__(self, dim, heads = 4, dim_head = 32, dropout = 0.):
+    def __init__(self, dim, heads = 4, dim_head = 32, scale = 8, dropout = 0.):
         super().__init__()
-        self.scale = dim_head ** -0.5
+        self.scale = scale
         self.heads = heads
         hidden_dim = dim_head * heads
 
@@ -208,6 +211,10 @@ class Attention(nn.Module):
 
         self.attn_dropout = nn.Dropout(dropout)
         self.to_qkv = nn.Linear(dim, hidden_dim * 3, bias = False)
+
+        self.q_scale = nn.Parameter(torch.ones(dim_head))
+        self.k_scale = nn.Parameter(torch.ones(dim_head))
+
         self.to_out = nn.Linear(hidden_dim, dim, bias = False)
 
     def forward(self, x):
@@ -216,9 +223,12 @@ class Attention(nn.Module):
         qkv = self.to_qkv(x).chunk(3, dim = -1)
         q, k, v = map(lambda t: rearrange(t, 'b n (h d) -> b h n d', h = self.heads), qkv)
 
-        q = q * self.scale
+        q, k = map(l2norm, (q, k))
 
-        sim = einsum('b h i d, b h j d -> b h i j', q, k)
+        q = q * self.q_scale
+        k = k * self.k_scale
+
+        sim = einsum('b h i d, b h j d -> b h i j', q, k) * self.scale
 
         attn = sim.softmax(dim = -1)
         attn = self.attn_dropout(attn)
@@ -318,6 +328,7 @@ class UViT(nn.Module):
         init_img_transform: callable = None,
         final_img_itransform: callable = None,
         patch_size = 1,
+        dual_patchnorm = False
     ):
         super().__init__()
 
@@ -333,20 +344,31 @@ class UViT(nn.Module):
 
         input_channels = channels
 
+        init_dim = default(init_dim, dim)
+        self.init_conv = nn.Conv2d(input_channels, init_dim, 7, padding = 3)
+
         # whether to do initial patching, as alternative to dwt
 
-        self.patchify = self.unpatchify = identity
+        self.unpatchify = identity
+
+        input_channels = channels * (patch_size ** 2)
+        needs_patch = patch_size > 1
+
+        if needs_patch:
+            if not dual_patchnorm:
+                self.init_conv = nn.Conv2d(channels, init_dim, patch_size, stride = patch_size)
+            else:
+                self.init_conv = nn.Sequential(
+                    Rearrange('b c (h p1) (w p2) -> b (c p1 p2) h w', p1 = patch_size, p2 = patch_size),
+                    LayerNorm(input_channels, normalize_dim = 1),
+                    nn.Conv2d(input_channels, init_dim, 1),
+                    LayerNorm(init_dim, normalize_dim = 1)
+                )
 
-        if patch_size > 1:
-            input_channels = channels * (patch_size ** 2)
-            self.patchify = nn.Conv2d(channels, input_channels, patch_size, stride = patch_size)
             self.unpatchify = nn.ConvTranspose2d(input_channels, channels, patch_size, stride = patch_size)
 
         # determine dimensions
 
-        init_dim = default(init_dim, dim)
-        self.init_conv = nn.Conv2d(input_channels, init_dim, 7, padding = 3)
-
         dims = [init_dim, *map(lambda m: dim * m, dim_mults)]
         in_out = list(zip(dims[:-1], dims[1:]))
 
@@ -417,7 +439,6 @@ class UViT(nn.Module):
 
     def forward(self, x, time):
         x = self.init_img_transform(x)
-        x = self.patchify(x)
 
         x = self.init_conv(x)
         r = x.clone()
